"""Base

Revision ID: 2478e08b1ba2
Revises: 
Create Date: 2020-05-24 10:20:18.384756

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '2478e08b1ba2'
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    pass


def downgrade():
    pass
